local _, addon = ...

function addon:PLAYER_LOGIN()
	-- Hook flex frame's onshow
	local function myFlexRaidFrame_OnShow(frame)
		self:CreateButtons(frame, GetNumFlexRaidDungeons, GetFlexRaidDungeonInfo, FlexRaidFrame_SetRaid)
		self:UpdateTooltips(frame)
	end
	FlexRaidFrame:HookScript("OnShow", myFlexRaidFrame_OnShow)

	-- Hook raid finder frame's onshow
	local function myRaidFinderFrame_OnShow(frame)
		self:CreateButtons(frame, GetNumRFDungeons, GetRFDungeonInfo, RaidFinderQueueFrame_SetRaid)
		self:UpdateTooltips(frame)
		self:UpdateQueueStatuses()
		self:RegisterEvent("LFG_UPDATE")
	end
	RaidFinderFrame:HookScript("OnShow", myRaidFinderFrame_OnShow)

	-- Hook raid finder frame's onhide
	local function myRaidFinderFrame_OnHide(frame)
		self:UnregisterEvent("LFG_UPDATE")
	end
	RaidFinderFrame:HookScript("OnHide", myRaidFinderFrame_OnHide)

	-- Our event dispatcher calls functions like addon[eventName](...), so point it to the right update function
	self.LFG_UPDATE = self.UpdateQueueStatuses
end
addon:RegisterEvent("PLAYER_LOGIN")

-- Creates the buttons. LFR and Flex use different functions and dropdown menus so take them as arguments
function addon:CreateButtons(parentFrame, DungeonAmountFunc, DungeonInfoFunc, SetDropdownMenuFunc)
	-- Ok, I still don't understand anything about the positioning and sizing of stuff in WoW, but the target frame is about 280'ish tall and buttons 32 and who gives a shit about margins and aaargh I'm going crazy /headexplode
	local scale = min(280 / (DungeonAmountFunc() * 32), 1)

	if not parentFrame.BossesKilledButtons then
		parentFrame.BossesKilledButtons = {}
	end
	local buttons = parentFrame.BossesKilledButtons

	for i = 1, DungeonAmountFunc() do
		local id, name = DungeonInfoFunc(i)
		local isAvailable, isAvailableToPlayer = IsLFGDungeonJoinable(id)

		 -- Only make a button if there's data for it, and it hasn't been already made. This gets called multiple times so it updates correctly when you open up more raids
		if isAvailable and isAvailableToPlayer and not buttons[id] and self.raidData[id] then
			local button = CreateFrame("CheckButton", parentFrame:GetName().."BossesKilledButton"..tostring(id), parentFrame, "SpellBookSkillLineTabTemplate")

			if parentFrame.lastButton then
				button:SetPoint("TOPLEFT", parentFrame.lastButton, "BOTTOMLEFT", 0, -17)
			else
				-- I don't like this fix for SocialTabs. It looks ugly, but it's good enough until (if) I figure out a more aesthetically pleasing solution
				button:SetPoint("TOPLEFT", parentFrame, "TOPRIGHT", (3 + (IsAddOnLoaded("SocialTabs") and ceil(32/scale) or 0)), -50)
			end

			button:SetScript("OnEnter", function(self) if self.tooltip then addon:ShowTooltip(self) end end)

			button:SetScript("OnClick", function(self)
				SetDropdownMenuFunc(self.dungeonID)

				-- This is to override the automatic highlighting when you click the button, while we want to use that to show queue status instead.
				-- I've no idea why simply overriding this OnClick and not doing a SetChecked doesn't disable the behavior.
				-- I probably shouldn't be using a CheckButton at all, but the SpellBookSkillLineTabTemplate looks pretty nice for the job.
				self:SetChecked(self.checked)
			end)
			button.checked = false

			button:SetScale(scale)
			button:Show()

			button.dungeonID = id
			button.dungeonName = name

			-- I just realised a CheckButton might already have it's own FontString, but uh... whatever. I'll fix it Soon(TM)
			local number = button:CreateFontString(button:GetName().."Number", "OVERLAY", "NumberFont_Outline_Med")
			button.number = number
			number:SetPoint("TOPLEFT", -3, 5, "TOPLEFT")
			number:SetPoint("BOTTOMRIGHT", 7, -5, "BOTTOMRIGHT")

			buttons[id] = button
			parentFrame.lastButton = button
		end
	end

	-- This might happen below level 90 I think, and at least when the next raid/expansion comes.
	if not next(buttons) then
		print("BossesKilled: No data for any raids. This addon might need an update. Currently only MoP raids have data.")
	end
end

function addon:UpdateTooltips(parentFrame)
	local buttons = parentFrame.BossesKilledButtons

	for id, button in pairs(buttons) do
		local numKilled = 0
		local tooltip = {{text = button.dungeonName}} -- Set up tooltip data with the dungeon name

		-- GetLFGDungeonNumEncounters returns the full set of bosses for the whole raid, while LFR and Flex raids
		-- are split into several separate raids, so get the amount of bosses and where to start from from our data file
		local index = self.raidData[id].startFrom
		local numEncounters = self.raidData[id].numEncounters

		for i = index, numEncounters + index - 1 do
			local bossName, _, isKilled = GetLFGDungeonEncounterInfo(id, i)

			if isKilled then
				table.insert(tooltip, {text = bossName, color = {RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b}})
				numKilled = numKilled + 1
			else
				table.insert(tooltip, {text = bossName, color = {GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b}})
			end
		end

		button.tooltip = tooltip

		local statusString = numKilled.."/"..numEncounters
		button.number:SetFormattedText(self.TextColorTable[statusString], statusString)
	end
end

function addon:ShowTooltip(button)
	GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
	for i = 1, #button.tooltip do
		tooltip = button.tooltip[i]
		if tooltip.color then
			GameTooltip:AddLine(tooltip.text, unpack(tooltip.color))
		else
			GameTooltip:AddLine(tooltip.text)
		end
	end
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("<Click to select this raid>")
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("(|c00ff0000Red|r for killed bosses, |c0000ff00green|r for alive ones,")
	GameTooltip:AddLine(" button lights up if you're queue'd for that raid)")
	GameTooltip:Show()
end

function addon:UpdateQueueStatuses()
	for id, button in pairs(RaidFinderFrame.BossesKilledButtons) do
		local mode = GetLFGMode(LE_LFG_CATEGORY_RF, id);
		if mode == "queued" or mode == "listed" or mode == "rolecheck" or mode == "suspended" then
			button:SetChecked(true)
			button.checked = true -- This is for the PostClick script above
		else
			button:SetChecked(false)
			button.checked = false
		end
	end
end

local select, modf, format, strmatch = select, math.modf, format, strmatch
-- gets passed a "x/y" STRING. No sanity checks so make sure the calling function feeds it the expected format.
-- the vararg gets passed color triples, eg. 0.0,1.0,0.0,  1.0,0.0,0.0 (green to red)
function addon:TextColorGradient(str_percent, ...)
	local num = select("#", ...)
	local low, high = strmatch(str_percent, "(%d+)/(%d+)")
	local percent = (low + 0) / (high + 0) -- implicit cast to number, cheaper than tonumber
	local r, g, b
	if percent >= 1 then
		r, g, b = select(num - 2, ...), select(num - 1, ...), select(num, ...)
		return format("|cff%02x%02x%02x%%s|r", r * 255, g * 255, b * 255)
	elseif percent <= 0 then
		r, g, b = ...
		return format("|cff%02x%02x%02x%%s|r", r * 255, g * 255, b * 255)
	end
	num = num / 3
	local segment, relperc = modf(percent * (num - 1))
	local r1, g1, b1, r2, g2, b2
	r1, g1, b1 = select((segment * 3) + 1, ...), select((segment * 3) + 2, ...), select((segment * 3) + 3, ...)
	r2, g2, b2 = select((segment * 3) + 4, ...), select((segment * 3) + 5, ...), select((segment * 3) + 6, ...)
	if not r2 or not g2 or not b2 then
		r, g, b = r1, g1, b1
		return format("|cff%02x%02x%02x%%s|r", r * 255, g * 255, b * 255)
	else
		r, g, b = r1 + (r2 - r1) * relperc,
		g1 + (g2 - g1) * relperc,
		b1 + (b2 - b1) * relperc
		return format("|cff%02x%02x%02x%%s|r", r * 255, g * 255, b * 255)
	end
end
-- use a memoization table so each x/y colorstring is only computed once and then does a simple lookup
addon.TextColorTable = setmetatable({}, {__index = function(t, k)
	local colorStr = addon:TextColorGradient(k, 0.0, 1.0, 0.0, 1.0, 0.0, 0.0) -- green to red
	rawset(t, k, colorStr)
	return colorStr
end})